@@ -100,7 +100,7 @@ static GStaticMutex iconv_cache_lock = G_STATIC_MUTEX_INIT;
  * Creates a new cache node, inserts it into the cache and increments
  * the cache size.
  *
- * Returns a pointer to the newly allocated cache node.
+ * Returns: a pointer to the newly allocated cache node.
  **/
 static IconvCacheNode *
 iconv_cache_node_new (const char *key, iconv_t cd)
@@ -154,6 +154,21 @@ iconv_cache_node_expire (Cache *cache, CacheNode *node)
 }
 
 
+static void
+iconv_open_node_free (gpointer key, gpointer value, gpointer user_data)
+{
+	iconv_t cd = (iconv_t) key;
+	IconvCacheNode *node;
+	
+	node = (IconvCacheNode *) cache_node_lookup (iconv_cache, value, FALSE);
+	g_assert (node);
+	
+	if (cd != node->cd) {
+		node->refcount--;
+		iconv_close (cd);
+	}
+}
+
 
 /**
  * g_mime_iconv_shutdown:
@@ -173,11 +188,13 @@ g_mime_iconv_shutdown (void)
 	fprintf (stderr, "The following %d iconv cache buckets are still open:\n", iconv_cache->size);
 	shutdown = 1;
 #endif
-	cache_free (iconv_cache);
-	iconv_cache = NULL;
 	
+	g_hash_table_foreach (iconv_open_hash, iconv_open_node_free, NULL);
 	g_hash_table_destroy (iconv_open_hash);
 	iconv_open_hash = NULL;
+	
+	cache_free (iconv_cache);
+	iconv_cache = NULL;
 }
 
 
@@ -215,7 +232,7 @@ g_mime_iconv_init (void)
  *
  * See the manual page for iconv_open(3) for further details.
  *
- * Returns a new conversion descriptor for use with g_mime_iconv() on
+ * Returns: a new conversion descriptor for use with g_mime_iconv() on
  * success or (iconv_t) %-1 on fail as well as setting an appropriate
  * errno value.
  **/
@@ -298,7 +315,7 @@ g_mime_iconv_open (const char *to, const char *from)
  *
  * See the manual page for iconv_close(3) for further details.
  *
- * Returns %0 on success or %-1 on fail as well as setting an
+ * Returns: %0 on success or %-1 on fail as well as setting an
  * appropriate errno value.
  **/
 int
