@@ -50,10 +50,10 @@ static int stream_flush (GMimeStream *stream);
 static int stream_close (GMimeStream *stream);
 static gboolean stream_eos (GMimeStream *stream);
 static int stream_reset (GMimeStream *stream);
-static off_t stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence);
-static off_t stream_tell (GMimeStream *stream);
+static gint64 stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence);
+static gint64 stream_tell (GMimeStream *stream);
 static ssize_t stream_length (GMimeStream *stream);
-static GMimeStream *stream_substream (GMimeStream *stream, off_t start, off_t end);
+static GMimeStream *stream_substream (GMimeStream *stream, gint64 start, gint64 end);
 
 
 static GMimeStreamClass *parent_class = NULL;
@@ -135,10 +135,10 @@ stream_read (GMimeStream *stream, char *buf, size_t len)
 		return -1;
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (off_t) len);
+		len = MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	fseek (fstream->fp, stream->position, SEEK_SET);
+	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 	
 	if ((nread = fread (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nread;
@@ -156,10 +156,10 @@ stream_write (GMimeStream *stream, const char *buf, size_t len)
 		return -1;
 	
 	if (stream->bound_end != -1)
-		len = MIN (stream->bound_end - stream->position, (off_t) len);
+		len = MIN (stream->bound_end - stream->position, (gint64) len);
 	
 	/* make sure we are at the right position */
-	fseek (fstream->fp, stream->position, SEEK_SET);
+	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 	
 	if ((nwritten = fwrite (buf, 1, len, fstream->fp)) > 0)
 		stream->position += nwritten;
@@ -216,17 +216,17 @@ stream_reset (GMimeStream *stream)
 	if (stream->position == stream->bound_start)
 		return 0;
 	
-	if (fseek (fstream->fp, stream->bound_start, SEEK_SET) == -1)
+	if (fseek (fstream->fp, (long) stream->bound_start, SEEK_SET) == -1)
 		return -1;
 	
 	return 0;
 }
 
-static off_t
-stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
+static gint64
+stream_seek (GMimeStream *stream, gint64 offset, GMimeSeekWhence whence)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
-	off_t real = stream->position;
+	gint64 real = stream->position;
 	FILE *fp = fstream->fp;
 	
 	g_return_val_if_fail (fstream->fp != NULL, -1);
@@ -244,7 +244,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 			 * we either don't know the offset of the end
 			 * of the stream and/or don't know if we can
 			 * seek past the end */
-			if (fseek (fp, offset, SEEK_END) == -1 || (real = ftell (fp)) == -1)
+			if (fseek (fp, (long) offset, SEEK_END) == -1 || (real = ftell (fp)) == -1)
 				return -1;
 		} else if (feof (fp) && stream->bound_end == -1) {
 			/* seeking backwards from eos (which happens
@@ -264,7 +264,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	if (stream->bound_end != -1 && real > stream->bound_end)
 		return -1;
 	
-	if (fseek (fp, real, SEEK_SET) == -1 || (real = ftell (fp)) == -1)
+	if (fseek (fp, (long) real, SEEK_SET) == -1 || (real = ftell (fp)) == -1)
 		return -1;
 	
 	stream->position = real;
@@ -272,7 +272,7 @@ stream_seek (GMimeStream *stream, off_t offset, GMimeSeekWhence whence)
 	return real;
 }
 
-static off_t
+static gint64
 stream_tell (GMimeStream *stream)
 {
 	return stream->position;
@@ -282,14 +282,14 @@ static ssize_t
 stream_length (GMimeStream *stream)
 {
 	GMimeStreamFile *fstream = (GMimeStreamFile *) stream;
-	off_t bound_end;
+	gint64 bound_end;
 	
 	if (stream->bound_start != -1 && stream->bound_end != -1)
 		return stream->bound_end - stream->bound_start;
 	
-	fseek (fstream->fp, 0, SEEK_END);
+	fseek (fstream->fp, (long) 0, SEEK_END);
 	bound_end = ftell (fstream->fp);
-	fseek (fstream->fp, stream->position, SEEK_SET);
+	fseek (fstream->fp, (long) stream->position, SEEK_SET);
 	
 	if (bound_end < stream->bound_start)
 		return -1;
@@ -298,7 +298,7 @@ stream_length (GMimeStream *stream)
 }
 
 static GMimeStream *
-stream_substream (GMimeStream *stream, off_t start, off_t end)
+stream_substream (GMimeStream *stream, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
@@ -320,13 +320,13 @@ stream_substream (GMimeStream *stream, off_t start, off_t end)
  * Note: The created GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
- * Returns a stream using @fp.
+ * Returns: a stream using @fp.
  **/
 GMimeStream *
 g_mime_stream_file_new (FILE *fp)
 {
 	GMimeStreamFile *fstream;
-	off_t start;
+	gint64 start;
 	
 	if ((start = ftell (fp)) == -1)
 		start = 0;
@@ -352,10 +352,10 @@ g_mime_stream_file_new (FILE *fp)
  * Note: The created GMimeStreamFile object will own the FILE pointer
  * passed in.
  *
- * Returns a stream using @fp with bounds @start and @end.
+ * Returns: a stream using @fp with bounds @start and @end.
  **/
 GMimeStream *
-g_mime_stream_file_new_with_bounds (FILE *fp, off_t start, off_t end)
+g_mime_stream_file_new_with_bounds (FILE *fp, gint64 start, gint64 end)
 {
 	GMimeStreamFile *fstream;
 	
@@ -374,7 +374,7 @@ g_mime_stream_file_new_with_bounds (FILE *fp, off_t start, off_t end)
  *
  * Gets whether or not @stream owns the backend FILE pointer.
  *
- * Returns %TRUE if @stream owns the backend FILE pointer or %FALSE
+ * Returns: %TRUE if @stream owns the backend FILE pointer or %FALSE
  * otherwise.
  **/
 gboolean
